'use strict';

var version = "2.3.8";

const toRegexStr = _ => _.join('|');
const COLORS = 'colors'; // 选填
const DIR_PATH = 'dirPath'; // 必填
const GENERATE = 'generate'; // 必填 生成地址
const UNIT = 'unit'; // 选填 单位
const IMPORTANT = 'important'; // 选填 单位
const GLOB_REG = 'globReg';
const V_TO_ANY = 'vToAny'; // px转rem配置
// 以下是项目配置 不从配置文件读
const EXT_NAME = 'extNames';
const MODIFY_RULES = 'modifyRules';
const MEDIA_QUERIES = 'mediaQueries';
// 以下是枚举

const JUSTIFY_CONTENT_ENUM = ['center', 'start', 'end', 'flex-start', 'flex-end', 'left', 'right', 'space-between', 'between', 'space-around', 'around', 'space-evenly', 'evenly', 'stretch', 'inherit', 'initial', 'unset', 'normal'];
const ALIGN_ITEMS_ENUM = ['baseline', 'center', 'end', 'flex-end', 'flex-start', 'inherit', 'initial', 'normal', 'self-end', 'self-start', 'start', 'stretch', 'unset'];

const CURSOR_ENUM = ['auto', 'default', 'none', 'context-menu', 'help', 'pointer', 'progress', 'wait', 'cell', 'crosshair', 'text', 'vertical-text', 'alias', 'copy', 'move', 'no-drop', 'not-allowed', 'e-resize', 'n-resize', 'ne-resize', 'nw-resize', 's-resize', 'se-resize', 'sw-resize', 'w-resize', 'ew-resize', 'ns-resize', 'nesw-resize', 'nwse-resize', 'col-resize', 'row-resize', 'all-scroll', 'zoom-in', 'zoom-out', 'grab', 'grabbing'];

const UNIT_ENUM = ['cm', 'mm', 'in', 'px', 'pt', 'pc', 'em', 'ex', 'ch', 'rem', 'vw', 'vh', 'vmin', 'vmax', 'p', 'v'];

const PSEUDO_ENUM = ['active', 'any-link', 'blank', 'checked', 'current', 'default', 'defined', 'disabled', 'drop', 'empty', 'enabled', 'first', 'first-child', 'first-of-type', 'fullscreen', 'future', 'focus', 'focus-visible', 'focus-within', 'host', 'hover', 'indeterminate', 'in-range', 'invalid', 'last-child', 'last-of-type', 'left', 'link', 'local-link', 'only-child', 'only-of-type', 'optional', 'out-of-range', 'past', 'placeholder-shown', 'read-only', 'read-write', 'required', 'right', 'root', 'scope', 'target', 'target-within', 'user-invalid', 'valid', 'visited'];

const GAP_ENUM = ['unset', 'initial', 'inherit', 'normal'];

const VERTICAL_ALIGN_ENUM = ['baseline', 'sub', 'super', 'text-top', 'text-bottom', 'middle', 'top', 'bottom', 'inherit', 'initial', 'unset'];
const PSEUDO_STR = toRegexStr(PSEUDO_ENUM);
const JUSTIFY_CONTENT_ENUM_STR = toRegexStr(JUSTIFY_CONTENT_ENUM);
const ALIGN_ITEMS_ENUM_STR = toRegexStr(ALIGN_ITEMS_ENUM);
const CURSOR_ENUM_STR = toRegexStr(CURSOR_ENUM);
const UNIT_ENUM_STR = toRegexStr(UNIT_ENUM);
const VERTICAL_ALIGN_STR = toRegexStr(VERTICAL_ALIGN_ENUM);
const GAP_ENUM_STR = toRegexStr(GAP_ENUM);
// 文件开头
let CSS_ANNOTATION = '';
CSS_ANNOTATION += '@charset "UTF-8";\n';
CSS_ANNOTATION += '/* https://github.com/macheteHot/css-generator-plugin/\n';
CSS_ANNOTATION += `  version: v${version}\n`;
CSS_ANNOTATION += '  this file will auto generate and update\n';
CSS_ANNOTATION += '  DO NOT EDIT THIS FILE!\n';
CSS_ANNOTATION += '  DO NOT EDIT THIS FILE!\n';
CSS_ANNOTATION += '  DO NOT EDIT THIS FILE!\n';
CSS_ANNOTATION += '*/\n\n';

const BASE_MEDIA_QUERY = {
  sm : '(min-width: 640px)',
  md : '(min-width: 768px)',
  lg : '(min-width: 1024px)',
  xl : '(min-width: 1280px)'
};

const BASE_MEDIA_QUERY_KEY = Object.keys(BASE_MEDIA_QUERY);
const NONNEGATIVE_NUMBER_REGEX_STR = '(0|([1-9]\\d*))(\\.\\d*[1-9])?';
const DIRECTION_MAP = new Map();
DIRECTION_MAP.set(undefined, ['']); // 全部
DIRECTION_MAP.set('x', ['left', 'right']);
DIRECTION_MAP.set('y', ['top', 'bottom']);
DIRECTION_MAP.set('t', ['top']);
DIRECTION_MAP.set('r', ['right']);
DIRECTION_MAP.set('b', ['bottom']);
DIRECTION_MAP.set('l', ['left']);
DIRECTION_MAP.set('top', ['top']);
DIRECTION_MAP.set('right', ['right']);
DIRECTION_MAP.set('bottom', ['bottom']);
DIRECTION_MAP.set('left', ['left']);

/**
 * order 220
 */
var alignItems = {
  regExp: new RegExp(`^align-items-(?<align>${ALIGN_ITEMS_ENUM_STR})$`),
  render ({ groups }) {
    const { align } = groups;
    return { name: 'alignItems', order: 220, css: [`align-items: ${align}`] }
  }
};

let programConfig = {
  [MODIFY_RULES]: {}
};
const runType = {
  vue: {
    [EXT_NAME] : ['vue'],
    reg        : /((?<=class=(["']))[\s\S]*?(?=\2))|((?<=class={)[\s\S]*?(?=}))/gi
  },
  react: {
    [EXT_NAME] : ['tsx', 'jsx'],
    reg        : /((?<=className=(["']))[\s\S]*?(?=\2))|((?<=className={)[\s\S]*?(?=}))/gi
  },
  'd-mini-program': { // 钉钉小程序
    [EXT_NAME] : ['axml'],
    reg        : /((?<=class=")|(?<=classname="))[\s\S]+?(?=")/gi
  },
  'wx-mini-program': { // 微信小程序
    [EXT_NAME] : ['wxml'],
    reg        : /((?<=class=")|(?<=classname="))[\s\S]+?(?=")/gi
  },
  html: { // 单纯 html
    [EXT_NAME] : ['html'],
    reg        : /((?<=class=(["']))[\s\S]*?(?=\2))/gi
  }
};
function getConfig$1 (str) {
  switch (str) {
    // 此处配置默认值
    case EXT_NAME:
      return runType[programConfig.type][EXT_NAME]
    case GLOB_REG:
      return runType[programConfig.type].reg
    case COLORS: // 默认空对象
      return programConfig[COLORS] || {}
    case UNIT: // 默认px
      return programConfig[UNIT] || 'px'
    case MEDIA_QUERIES: // 默认px
      return programConfig[MEDIA_QUERIES] || {}
    case IMPORTANT: // 默认添加！important 设置默认值
      return programConfig[IMPORTANT] === undefined ? true : programConfig[IMPORTANT]
    default: // dirPath generate  等项目配置
      return programConfig[str]
  }
}

function setConfig (config) {
  programConfig = config;
}

function getUnit (number, str) {
  if (Number(number) === 0) {
    return ''
  }
  if (str === 'p') {
    return '%'
  }
  if (!str) {
    return programConfig[UNIT] || 'px'
  }
  return str
}

function isFunction (payload) {
  return Object.prototype.toString.call(payload) === '[object Function]'
}

function isObject (payload) {
  return Object.prototype.toString.call(payload) === '[object Object]'
}

function groupBy (array, name) {
  const groups = {};
  array.forEach(function (o) {
    const group = JSON.stringify(o[name]);
    groups[group] = groups[group] || [];
    groups[group].push(o);
  });
  return Object.keys(groups).map(function (group) {
    return groups[group]
  })
}

function getDirectionOrder (order, direction) {
  if (!direction) { return order }
  switch (direction) {
    case 'x':
      return order + 10
    case 'y':
      return order + 20
    case 't':
      return order + 30
    case 'b':
      return order + 40
    case 'r':
      return order + 50
    case 'l':
      return order + 60
  }
}

function v2any (number) {
  const { rootValue = 16, unitPrecision = 5, minPixelValue = 1 } = getConfig$1(V_TO_ANY);

  const pixels = parseFloat(number);
  if (pixels < minPixelValue) return number

  const unitPrecisionTimes = Math.pow(10, unitPrecision);
  const v = Math.floor(pixels / rootValue * unitPrecisionTimes) / unitPrecisionTimes;

  return v
}

/**
 * order 520 460 + 60
 */

const getOrder = direction => getDirectionOrder(460, direction);

function getCss (direction, num, unit) {
  return DIRECTION_MAP
    .get(direction)
    .reduce((t, c) => {
      if (c) {
        return [...t, `border-${c}-width: ${num}${unit}`, `border-${c}-style: solid`, `border-${c}-color: rgba(0,0,0,1)`]
      } else {
        return [...t, `border-width: ${num}${unit}`, 'border-style: solid', 'border-color: rgba(0,0,0,1)']
      }
    }, [])
}

var border = {
  regExp: new RegExp(`^(border|border-width|border-w)-((?<direction>[trblxy])-)?(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { direction, num, unit } = groups;
    return {
      name  : direction ? `border-${direction}` : 'border',
      order : getOrder(direction),
      num,
      css   : getCss(direction, num, unit)
    }
  }
};

/**
 * order 530
 */

var borderRadius = {
  regExp: new RegExp(`^(border-radius|br)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return {
      name  : 'borderRadius',
      order : 530,
      num,
      css   : [`border-radius: ${num}${unit}`]
    }
  }
};

/**
 * order 550
 */

var borderStyle = {
  regExp: /^border-style-(?<value>none|hidden|dotted|dashed|solid|double|groove|ridge|inset|outset|inherit)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'borderStyle',
      order : 550,
      css   : [`border-style: ${value}`]
    }
  }
};

/**
 * order 560
 */

var boxSizing = {
  regExp: /^box-sizing-(?<value>content-box|border-box)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'boxSizing',
      order : 560,
      css   : [`box-sizing: ${value}`]
    }
  }
};

/**
 * order 540
 */

var circle = {
  regExp: /^circle$/,
  render () {
    return {
      name  : 'circle',
      order : 540,
      css   : ['border-radius: 50%']
    }
  }
};

const colorStore = () => ({
  red         : '#f00',
  white       : '#fff',
  black       : '#000',
  blue        : '#00f',
  transparent : 'transparent',
  ...getConfig$1(COLORS)
});

function getColors () {
  return colorStore()
}

function getColorsKey () {
  return Object.keys(colorStore())
}

function radix16 (value) {
  return parseInt(value, 16)
}

function textToRgbText (str, opacity = 1) {
  const hex = /^#?([a-fA-F0-9]{8}|[a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/.test(str) // is hex text or word
    ? str.replace(/^#/, '')
    : colorStore()[str].replace(/^#/, '');
  if (hex === 'transparent') {
    return 'transparent'
  }
  // more use
  if (hex.length === 6) {
    const reg = /[a-fA-F0-9]{2}/g;
    return 'rgba(' + hex
      .match(reg)
      .map(radix16)
      .join(',') +
      `,${opacity})`
  }
  if (hex.length === 3) {
    return 'rgba(' + hex
      .split('')
      .map(x => radix16(x.repeat(2)))
      .join(',') +
      `,${opacity})`
  }
  if (hex.length === 8) {
    const reg = /[a-fA-F0-9]{2}/g;
    let [r, g, b, a] = hex.match(reg);
    a = Number(Number(Math.round(parseInt(a, 16)) / 255).toFixed(2));
    return 'rgba(' + [r, g, b]
      .map(radix16)
      .join(',') +
      `,${a})`
  }
  return ''
}

/**
 * order Infinity
 */

var color = {
  regExp: () => new RegExp(
    `^(?<type>color|c|text|bg|background|border-color|border-c)-(?<color>(#?([a-fA-F0-9]{8}$|[a-fA-F0-9]{6}|[a-fA-F0-9]{3}))|${getColorsKey().join('|')})(-(?<opacity>1|([1-9]\\d?)))?$`),
  render ({ groups }) {
    let { type, color, opacity } = groups;
    opacity = opacity === undefined ? 1 : (opacity * 0.01).toFixed(2);
    color = textToRgbText(color, opacity); // rgba(xxxx) or transparent
    let prefix = '';
    switch (type) {
      case 'c':
      case 'color':
      case 'text':
        prefix = 'color';
        break
      case 'bg':
      case 'background':
        prefix = 'background-color';
        break
      case 'border-c':
      case 'border-color':
        prefix = 'border-color';
        break
      default:
        prefix = type;
        break
    }
    return { name: 'color', order: Infinity, css: [`${prefix}: ${color}`] }
  }
};

var columnGap = {
  regExp: new RegExp(`^c(olumn)?-gap-(((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|(?<value>${GAP_ENUM_STR}))$`),
  render ({ groups }) {
    let { num = Infinity, unit, value } = groups;
    if (!value) { value = num + unit; }
    return { name: 'column-gap', order: 582, num, css: [`column-gap: ${value}`] }
  }
};

/**
 * order 340
 */

var cursor = {
  regExp: new RegExp(`^cursor-(?<value>${CURSOR_ENUM_STR})$`),
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'cursor',
      order : 340,
      css   : [`cursor: ${value}`]
    }
  }
};

/**
 * order 290
 */

var display = {
  regExp: /^(display|d)-(?<value>none|inline|block|inline-block|flex|contents)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'display',
      order : Infinity,
      css   : [`display: ${value}`]
    }
  }
};

/**
 * order 280
 */

var flexBasis = {
  regExp: new RegExp(`^flex-basis-(?<value>((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|initial|inherit|auto)$`),
  render ({ groups }) {
    let { value, num, unit } = groups;
    if (num) {
      value = `${num}${unit}`;
    }
    return {
      name  : 'flexBasis',
      order : 280,
      css   : [`flex-basis: ${value}`]
    }
  }
};

/**
 * order 230
 */
var flexDirection = {
  regExp: /^(flex-direction|flex)-(?<value>row|row-reverse|column|column-reverse)$/,
  render ({ groups }) {
    const { value } = groups;
    return { name: 'flexDirection', order: 230, css: [`flex-direction: ${value}`] }
  }
};

/**
 * order 200
 */
var flexJustAli = {
  regExp: new RegExp(`^flex-(?<justify>${JUSTIFY_CONTENT_ENUM_STR})-(?<align>${ALIGN_ITEMS_ENUM_STR})$`),
  render ({ groups }) {
    let { justify, align } = groups;
    if (justify === 'between') {
      justify = 'space-between';
    }
    if (justify === 'around') {
      justify = 'space-around';
    }
    if (justify === 'evenly') {
      justify = 'space-evenly';
    }
    return {
      name  : 'flexJustAli',
      order : 200,
      css   : ['display: flex', `justify-content: ${justify}`, `align-items: ${align}`]
    }
  }
};

/**
 * order 250
 */

var flexNum = {
  regExp: /^flex-(?<value>null|auto|none|(0|[1-9]\d*))$/,
  render ({ groups }) {
    const { value } = groups;
    return { name: 'flex', order: 250, css: [`flex: ${value}`] }
  }
};

/**
 * order 260-270
 */
var flexShrinkAndGrow = {
  regExp: /^flex-(?<type>shrink|grow)-(?<value>(0|[1-9]\d*)|initial|inherit)$/,
  render ({ groups }) {
    const { type, value } = groups;
    return {
      name  : type === 'shrink' ? 'flexShrink' : 'flexGrow',
      order : type === 'shrink' ? 260 : 270,
      css   : [`flex-${type}: ${value}`]
    }
  }
};

/**
 * order 240
 */
var flexWrap = {
  regExp: /^flex-wrap-(?<value>inherit|initial|nowrap|wrap|wrap-reverse)$/,
  render ({ groups }) {
    const { value } = groups;
    return { name: 'flexDirection', order: 240, css: [`flex-wrap: ${value}`] }
  }
};

/**
 * order 370
 */

var fontSize = {
  regExp: new RegExp(`^(font-size|fs)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'fontSize', order: 370, num, css: [`font-size: ${num}${unit}`] }
  }
};

/**
 * order 360
 */
var fontWeight = {
  regExp: /^(font-weight|fw)-(?<value>[1-9]00|bold|bolder|inherit|initial|lighter|normal|unset)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'fontWeight',
      order : 360,
      css   : [`font-weight: ${value}`]
    }
  }
};

var gap = {
  regExp: new RegExp(`^gap-(((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|(?<value>${GAP_ENUM_STR}))$`),
  render ({ groups }) {
    let { num = Infinity, unit, value } = groups;
    if (!value) { value = num + unit; }
    return { name: 'gap', order: 580, num, css: [`column-gap: ${value}`, `row-gap: ${value}`] }
  }
};

/**
 * order 20
 */

var height = {
  regExp: new RegExp(`^(h|height)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return {
      name  : 'height',
      order : 20,
      num,
      css   : [`height: ${num}${unit}`]
    }
  }
};

/**
 * order 210
 */
var justifyContent = {
  regExp: new RegExp(`^justify-content-(?<justify>${JUSTIFY_CONTENT_ENUM_STR})$`),
  render ({ groups }) {
    const { justify } = groups;
    return { name: 'justifyContent', order: 210, css: [`justify-content: ${justify}`] }
  }
};

/**
 * order 410
 */

var letterSpacing = {
  regExp: new RegExp(`^letter-spacing-(?<isMinus>m-)?(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    let { isMinus, num, unit } = groups;
    if (isMinus) {
      num = 0 - num;
    }
    return { name: 'letterSpacing', order: 410, num, css: [`letter-spacing: ${num}${unit}`] }
  }
};

/**
 * order 330
 */

var lineHeight = {
  regExp: new RegExp(`^(lh|line-height)-(?<value>((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|normal|unset|inherit|initial)$`),
  render ({ groups }) {
    const { value, num = Infinity, unit } = groups;
    const base = { name: 'lineHeight', order: 330 };
    if (num !== Infinity) {
      return { ...base, num, css: [`line-height: ${num}${unit}`] }
    }
    return { ...base, num, css: [`line-height: ${value}`] }
  }
};

/**
 * 50 - 180 order
 */

function getConfig (type, direction) {
  let order;
  let name = '';
  if (type === 'm' || type === 'margin') {
    order = getDirectionOrder(50, direction);
    name += 'margin';
  }
  // 单项加 60 order m-l-10 最大为50 + 60 padding 从 120 起
  if (type === 'p' || type === 'padding') {
    order = getDirectionOrder(110, direction);
    name += 'padding';
  }
  if (direction) {
    name += `-${direction}`;
  }
  return { name, order }
}

var marginAndPadding = {
  regExp: new RegExp(`^(?<type>m|margin|p|padding)-((?<direction>[trblxy])-)?((?<auto>auto)|(?<isMinus>m-)?(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)$`),
  render ({ groups }) {
    let { type, direction, isMinus, num, unit, auto } = groups;
    if (auto) {
      unit = '';
      num = 'auto';
    }
    // if has auto never has isMinus
    if (isMinus) {
      num = 0 - num;
    }
    const baseConfig = getConfig(type, direction);
    if (type === 'm') {
      type = 'margin';
    }
    if (type === 'p') {
      type = 'padding';
    }
    return {
      ...baseConfig,
      num,
      css: DIRECTION_MAP
        .get(direction)
        .reduce((t, c) =>
          [...t, c ? `${type}-${c}: ${num}${unit}` : `${type}: ${num}${unit}`], [])
    }
  }
};

/**
 * order 31
 */

var maxHeight = {
  regExp: new RegExp(`^(max-h|max-height)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'max-height', order: 31, num, css: [`max-height: ${num}${unit}`] }
  }
};

/**
 * order 31
 */

var maxWidth = {
  regExp: new RegExp(`^(max-w|max-width)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'max-width', order: 31, num, css: [`max-width: ${num}${unit}`] }
  }
};

/**
 * order 40
 */

var minHeight = {
  regExp: new RegExp(`^(min-h|min-height)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'min-height', order: 40, num, css: [`min-height: ${num}${unit}`] }
  }
};

/**
 * order 41
 */

var minWidth = {
  regExp: new RegExp(`^(min-w|min-width)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'min-width', order: 41, num, css: [`min-width: ${num}${unit}`] }
  }
};

/**
 * order Infinity
 */

var objectFit = {
  regExp: /^(object-fit)-(?<value>fill|contain|cover|none|scale-down|inherit|initial|revert|unset)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'objectFit',
      order : Infinity,
      css   : [`object-fit: ${value}`]
    }
  }
};

/**
 * order Infinity
 */
var opacity = {
  regExp: /^opacity-(?<value>([1-9]?\d|100))$/,
  render ({ groups }) {
    const { value } = groups;
    return { name: 'opacity', order: Infinity, num: value, css: [`opacity: ${Number((value / 100).toFixed(2))}`] }
  }
};

/**
 * order 310
 */

var orientation = {
  regExp: new RegExp(`^(?<direction>[trbl]|top|right|bottom|left)-(?<isMinus>m-)?(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    let { direction, isMinus, num, unit } = groups;
    if (isMinus) {
      num = 0 - num;
    }
    // is only t r b l
    if (direction.length === 1) {
      direction = DIRECTION_MAP.get(direction)[0];
    }

    return { name: 'orientation', order: 310, num, css: [`${direction}: ${num}${unit}`] }
  }
};

/**
 * order 380 - 400
 */

var overflow = {
  regExp: /^overflow(-(?<direction>[xy]))?-(?<value>hidden|auto|visible|scroll|inherit)$/,
  render ({ groups }) {
    const { direction, value } = groups;
    const base = { name: 'overflow' };
    if (!direction) {
      return { ...base, order: 380, css: [`overflow: ${value}`] }
    }
    if (direction === 'x') {
      return { ...base, order: 390, css: [`overflow-x: ${value}`] }
    }
    if (direction === 'y') {
      return { ...base, order: 400, css: [`overflow-y: ${value}`] }
    }
  }
};

/**
 * order 300
 */

var position = {
  regExp: /^position-(?<value>static|relative|sticky|unset|absolute|fixed|inherit|initial)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'position',
      order : 300,
      css   : [`position: ${value}`]
    }
  }
};

var rowGap = {
  regExp: new RegExp(`^r(ow)?-gap-(((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|(?<value>${GAP_ENUM_STR}))$`),
  render ({ groups }) {
    let { num = Infinity, unit, value } = groups;
    if (!value) { value = num + unit; }
    return { name: 'row-gap', order: 581, num, css: [`row-gap: ${value}`] }
  }
};

/**
 * order 50
 */

var square = {
  className : 'square',
  regExp    : new RegExp(`^square-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'square', order: 50, num, css: [`width: ${num}${unit}`, `height: ${num}${unit}`] }
  }
};

/**
 * order 320
 */

var textAlign = {
  regExp: /^(text-align|text)-(?<value>start|end|left|right|center|justify|match-parent)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'textAlign',
      order : 320,
      css   : [`text-align: ${value}`]
    }
  }
};

/**
 * order 440
 */

var textAlignLast = {
  regExp: /^(text-align-last|text-last)-(?<value>auto|left|right|center|justify|start|end|initial|inherit)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'textAlignLast',
      order : 440,
      css   : [`text-align-last: ${value}`]
    }
  }
};

/**
 * order 420
 */

var textDecoration = {
  regExp: /^(text-decoration|text)-(?<value>none|underline|overline|line-through|blink|inherit)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'textDecoration',
      order : 420,
      css   : [`text-decoration: ${value}`]
    }
  }
};

/**
 * order 450
 */

var textEllipsis = {
  regExp: /^(text-)?ellipsis(-(?<value>[1-9]\d*))?$/,
  render ({ groups }) {
    let { value } = groups;
    const base = { name: 'ellipsis', order: 450 };
    if (Number(value) === 1) {
      value = undefined; // 和没写是一样的
    }
    if (value === undefined) {
      return {
        ...base,
        num : 0,
        css : [
          'overflow: hidden',
          'text-overflow: ellipsis',
          'white-space: nowrap'
        ]
      }
    } else {
      return {
        ...base,
        num : value,
        css : [
          'overflow: hidden',
          'text-overflow: ellipsis',
          'display: -webkit-box',
          `-webkit-line-clamp: ${value}`,
          '-webkit-box-orient: vertical'
        ]
      }
    }
  }
};

/**
 * order 430
 */

var userSelect = {
  regExp: /^(user-)?select-(?<value>none|auto|text|all|contain|element)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'userSelect',
      order : 430,
      css   : [`user-select: ${value}`]
    }
  }
};

/**
 * order 570
 */

var verticalAlign = {
  regExp: new RegExp(`^vertical-align-(?<value>((?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?)|${VERTICAL_ALIGN_STR})$`),
  render ({ groups }) {
    let { value, num, unit } = groups;
    if (num) {
      value = `${num}${unit}`;
    }
    return {
      name  : 'verticalAlign',
      order : 570,
      css   : [`vertical-align: ${value}`]
    }
  }
};

/**
 * order 590
 */
const list = ['visible', 'hidden', 'collapse', 'inherit', 'initial', 'revert', 'unset'];
var visibility = {
  regExp: new RegExp(`^visibility-(?<value>${toRegexStr(list)})$`),
  render ({ groups }) {
    const { value } = groups;
    return { name: 'visibility', order: 590, css: [`visibility-${value}: ${value}`] }
  }
};

/**
 * order 10
 */

var width = {
  regExp: new RegExp(`^(w|width)-(?<num>${NONNEGATIVE_NUMBER_REGEX_STR})(?<unit>${UNIT_ENUM_STR})?$`),
  render ({ groups }) {
    const { num, unit } = groups;
    return { name: 'width', order: 10, num, css: [`width: ${num}${unit}`] }
  }
};

/**
 * order 350
 */

var wordBreak = {
  regExp: /^word-break-(?<value>normal|break-all|keep-all|break-word|inherit|initial|unset)$/,
  render ({ groups }) {
    const { value } = groups;
    return {
      name  : 'wordBreak',
      order : 350,
      css   : [`word-break: ${value}`]
    }
  }
};

/**
 * order 190
 */
var zIndex = {
  regExp: /^z-index-(?<isMinus>m-)?(?<value>0|[1-9]\d*)$/,
  render ({ groups }) {
    let { isMinus, value } = groups;
    if (isMinus) {
      value = 0 - value;
    }
    return { name: 'zIndex', order: 190, num: value, css: [`z-index: ${value}`] }
  }
};

var rules = /*#__PURE__*/Object.freeze({
  __proto__: null,
  alignItems: alignItems,
  border: border,
  borderRadius: borderRadius,
  borderStyle: borderStyle,
  boxSizing: boxSizing,
  circle: circle,
  color: color,
  columnGap: columnGap,
  cursor: cursor,
  display: display,
  flexBasis: flexBasis,
  flexDirection: flexDirection,
  flexJustAli: flexJustAli,
  flexNum: flexNum,
  flexShrinkAndGrow: flexShrinkAndGrow,
  flexWrap: flexWrap,
  fontSize: fontSize,
  fontWeight: fontWeight,
  gap: gap,
  height: height,
  justifyContent: justifyContent,
  letterSpacing: letterSpacing,
  lineHeight: lineHeight,
  marginAndPadding: marginAndPadding,
  maxHeight: maxHeight,
  maxWidth: maxWidth,
  minHeight: minHeight,
  minWidth: minWidth,
  objectFit: objectFit,
  opacity: opacity,
  orientation: orientation,
  overflow: overflow,
  position: position,
  rowGap: rowGap,
  square: square,
  textAlign: textAlign,
  textAlignLast: textAlignLast,
  textDecoration: textDecoration,
  textEllipsis: textEllipsis,
  userSelect: userSelect,
  verticalAlign: verticalAlign,
  visibility: visibility,
  width: width,
  wordBreak: wordBreak,
  zIndex: zIndex
});

let preArray = [];
const queryObj = { };

function pushPreObj (obj) {
  return preArray.push(obj)
}

function pushQuery (key, obj) {
  // if queryObj has key then push else set key is Array
  if (Object.prototype.hasOwnProperty.call(queryObj, key)) {
    queryObj[key].push(obj);
  } else {
    queryObj[key] = [obj];
  }
}

const isImportant = () => getConfig$1(IMPORTANT);
function getCssSingle ({ classStr, css, pseudo }) {
  classStr = classStr.replace(/([@:#.])/g, '\\$1');
  if (pseudo) {
    classStr = classStr + `:${pseudo}`;
  }
  return css.reduce((t, c) => t + (isImportant() ? `${c} !important; ` : `${c}; `), `.${classStr}{ `) + '}'
}

function sortCss (a, b) {
  if (a !== undefined && b !== undefined) {
    return Number(a.num) - Number(b.num)
  } else {
    return 0
  }
}

function renderArray (array) {
  let cssStr = '';
  const cssObject = groupBy(array.sort((a, b) => a.order - b.order), 'name');
  for (const key in cssObject) {
    if (Object.prototype.hasOwnProperty.call(cssObject, key)) {
      cssStr += `/* ${cssObject[key][0].name || 'unknow name'} order ${cssObject[key][0].order} */\n`;
      cssStr += cssObject[key]
        .sort(sortCss)
        .map(getCssSingle)
        .join('\n');
      cssStr += '\n\n';
    }
  }
  return cssStr
}

function renderCss () {
  let cssStr = '';
  cssStr += renderArray(preArray);
  const queryConfigObj = { ...BASE_MEDIA_QUERY, ...getConfig$1(MEDIA_QUERIES) };
  for (const key in queryObj) {
    if (Object.prototype.hasOwnProperty.call(queryObj, key)) {
      cssStr += `@media ${queryConfigObj[key]}{\n`;
      cssStr += renderArray(queryObj[key]);
      cssStr += '}';
    }
  }
  return cssStr
}

const modifyUtils = { textToRgbText, getColorsKey, getColors, UNIT_ENUM_STR, NONNEGATIVE_NUMBER_REGEX_STR, DIRECTION_MAP };
const cssSet = new Set();
const handleCssPipe = new Set();

function filterClassNames (sourceStr) {
  // cssSet.clear() // 清空set
  // clearPreArray() // 清空预编译
  const classNameList = sourceStr.match(getConfig$1(GLOB_REG));
  if (classNameList) {
    classNameList.forEach(hasClassNameStr => {
      // 替换我们规则中不会出现的字符 替换成空格 注意前后必须有空格 可能导致拼接合法 会多生成几条 无所谓
      const className = hasClassNameStr.replace(/[^a-zA-Z0-9-@:#.]/g, ' ');
      className.split(' ').forEach(filterClass);
    });
  }
  return ''
}

function filterClass (classStr) {
  if (cssSet.has(classStr)) {
    return null
  }
  let query; let pseudo; let source = classStr;
  const queryNames = [...BASE_MEDIA_QUERY_KEY, ...Object.keys(getConfig$1(MEDIA_QUERIES))];
  const v2anyConfig = getConfig$1(V_TO_ANY);
  const isV2any = isObject(v2anyConfig);

  if (/[@:]/.test(classStr)) {
    const queryAndPesudoRegex = new RegExp(`^(?:(?<query>${queryNames.join('|')})@)?(?:(?<pseudo>${PSEUDO_STR}):)?(?<source>[^:@]+)$`);
    const res = classStr.match(queryAndPesudoRegex);
    if (!res) {
      return null
    }
    const { groups = null } = res;
    if (!groups) {
      return null
    }
    ({ query, pseudo, source } = groups);
  }

  cssSet.add(classStr);
  const ruleList = Object.values({ ...rules, ...getConfig$1(MODIFY_RULES) });
  for (let i = 0; i < ruleList.length; i++) {
    let rule = ruleList[i];
    rule = isFunction(rule) ? rule(modifyUtils) : rule;
    const reg = isFunction(rule.regExp) ? rule.regExp() : rule.regExp;
    const res = source.match(reg);
    if (res !== null) {
      const { unit, num } = res.groups || {};
      const unit1 = getUnit(num, unit);

      if (isObject(res.groups)) {
        if (isV2any && unit1 === 'v' && num) {
          Object.assign(res.groups, {
            num  : v2any(num),
            unit : v2anyConfig.unit
          });
        } else {
          Object.assign(res.groups, { unit: unit1 });
        }
      }

      let renderResult = rule.render(res);

      handleCssPipe.forEach(handle => {
        renderResult = handle(renderResult);
      });

      const params = {
        classStr,
        ...renderResult,
        pseudo
      };

      if (query) {
        pushQuery(query, params);
      } else {
        pushPreObj(params);
      }
      break
    }
  }
}

const { performance } = require('perf_hooks');
const fs = require('fs');
const path = require('path');
const glob = require('glob');
const shelljs = require('shelljs');
let hotReload = false;

let startTime = 0;
let endTime = 0;
const setTimeStart = () => {
  startTime = performance.now();
};
const setTimeEnd = () => {
  endTime = performance.now();
};

const getUseTime = () => (endTime - startTime).toFixed(2);

function getAllFileClassStr () {
  const globSyncStr = getConfig$1(EXT_NAME).join('|');
  const files = glob.sync(path.join(process.cwd(), `./${getConfig$1(DIR_PATH)}/**/*.@(${globSyncStr})`));
  return files.reduce((t, c) => t + fs.readFileSync(path.resolve(c), 'utf8'), '')
}

function wirteToFile () {
  const cssFilePath = path.resolve(getConfig$1(GENERATE));
  const cssDirPath = path.resolve(getConfig$1(GENERATE), '..');
  if (!fs.existsSync(cssDirPath)) {
    shelljs.mkdir('-p', cssDirPath);
  }
  fs.writeFileSync(cssFilePath, `${CSS_ANNOTATION}${renderCss()}\n`);
}

function getFilePath (str) {
  return path.resolve(process.cwd(), str)
}

function logUseTime () {
  console.log(`css generator ${hotReload ? 'reload' : 'init'} done use time ${getUseTime()}ms`);
}

function init () {
  setTimeStart();
  filterClassNames(getAllFileClassStr());
  wirteToFile();
  setTimeEnd();
  logUseTime();
}

function readConfigFile () {
  let options = null;
  if (fs.existsSync(getFilePath('css.generator.config.js'))) {
    options = require(getFilePath('css.generator.config.js'));
  } else {
    throw new Error('you dont have any config!!! see https://github.com/macheteHot/css-generator-plugin/blob/master/README.md')
  }
  return options
}

function hotReloadFn (txt) {
  hotReload = true;
  setTimeStart();
  filterClassNames(txt);
  wirteToFile();
  setTimeEnd();
  logUseTime();
}

module.exports = function vitePlugin () {
  const gcssConfig = readConfigFile();
  return {
    name: 'vite-plugin',
    config (config) {
      setConfig(gcssConfig);
      init();
      return config
    },
    async handleHotUpdate ({ file, read }) {
      const [, extName] = file.match(/(?<=\.)([\w\d]+)$/);
      if (!getConfig$1(EXT_NAME).includes(extName)) { return null }
      hotReloadFn(await read());
    }

  }
};
